import inspect
from typing import List


def ams_math_package() -> str:
    """
    Returns the code needed to add in preamble to type math formula in latex
    """
    return inspect.cleandoc(r"""
    \usepackage{amsmath}
    \usepackage{amsfonts}
    \usepackage{amssymb}
    """)


def two_column_header(left: str, right: str) -> str:
    """
    Create exam header with two column

    :param left: left text
    :param right: right text
    :return: Latex code
    """
    return inspect.cleandoc(rf"""
    \begin{{tabular}}{{ *{{2}}{{ p{{ \dimexpr0.5\linewidth-2\tabcolsep\relax }} }} }}
    {left} & {right}
    \end{{tabular}}
    """)


def bold_title(text: str) -> str:
    """
    Create bold title in exam header

    :return: latex code
    """
    return inspect.cleandoc(rf"""
    \begin{{center}}
    \textbf {{ {{\Large {text} }} }}
    \end{{center}}
    """)


def geometry_package(top: float, bottom: float, left: float, right: float) -> str:
    """
    Generate latex code to add geometry package

    :param top: top margin
    :param bottom: bottom margin
    :param left: left margin
    :param right: right margin
    """
    return inspect.cleandoc(rf"""
        \usepackage[left = {left}cm, right = {right}cm, top = {top}cm, bottom = {bottom}cm]{{geometry}}
    """)


def add_multiple_package(package_list: List[str]) -> str:
    """
    Generate latex code to add multiple package to preamble

    :param package_list: List of package to add in preamble
    """
    usepackage_command_list = []
    for package in package_list:
        usepackage_command_list.append(rf"""\usepackage{{{package}}}""")

    return "\n".join(usepackage_command_list)



class Answer:
    """
    This class is used to store 1 answer in a question.
    """
    def __init__(self, answer_key: str, answer: str, is_true_answer=False):
        """
        This method initializes an Answer object.

        :param answer_key: Answer
        :param answer: Answer to the question.
        :param is_true_answer: If the answer is correct,the value is True, otherwise False.
        """
        self.answer_key = answer_key
        self.answer: str = answer
        self.is_true_answer: bool = is_true_answer



import random
from typing import List

from pytexexam.question import Question


class Exam:
    """
    This class represents an exam.
    """
    def __init__(self, question_list: List[Question]):
        self.question_list = question_list
        """List of questions in the exam"""

    def shuffle_question(self, seed: int = None):
        """
        This method allows to shuffle all the questions in the exam.
        :param seed: random seed
        """
        if seed is not None:
            random.seed(seed)
        random.shuffle(self.question_list)



from enum import Enum
from typing import List

from pytexexam.latexexam import LatexExamPaper, LatexExamAnswer, LatexExamSolution
from pytexexam.latexpaper import LatexPaper
from pytexexam.question import Question
import random


class LatexExamBuilder:
    """Builder class to create exam,answer and solution paper"""
    def __init__(self):
        self.preamble = ""
        self.header = ""
        self.questions: List[Question] = list()
        self.footer = ""
        self.export_type: ExamExportType = ExamExportType.TEX

    def create_exam(self, file_dir: str):
        """Create exam paper"""
        exam = LatexExamPaper()
        exam.preamble = self.preamble
        exam.header = self.header
        exam.questions = self.questions
        exam.footer = self.footer
        self.__export(exam, file_dir)

    def create_answer(self, file_dir: str):
        """Create answer key paper"""
        exam = LatexExamAnswer()
        exam.preamble = self.preamble
        exam.header = self.header
        exam.questions = self.questions
        exam.footer = self.footer
        self.__export(exam, file_dir)

    def create_solution(self, file_dir: str):
        """Create solution paper"""
        exam = LatexExamSolution()
        exam.preamble = self.preamble
        exam.header = self.header
        exam.questions = self.questions
        exam.footer = self.footer
        self.__export(exam, file_dir)

    def __export(self, paper: LatexPaper, file_dir: str):
        """Export exam to .tex file or pdf file"""
        if self.export_type == ExamExportType.TEX:
            paper.export_tex_file(file_dir)
        elif self.export_type == ExamExportType.PDF:
            paper.export_pdf_file(file_dir)
        else:
            paper.export_tex_file(file_dir)

    def add_question(self, question: str, answer: List[str], true_answer: str, answer_column: int, solution: str = ""):
        """
        Add question to exam

        :param question: question stem
        :param answer: List of answers in order A, B, C, D
        :param true_answer: Answer key of true answer.
        :param answer_column: Number of columns used to present the answer
        :param solution: Solution for this question
        """
        question = Question(question, answer, true_answer, solution, answer_column)
        self.questions.append(question)

    def shuffle_all_question(self, seed: int = None):
        """
        Shuffle all question in exam

        :param seed: random seed
        """
        if seed is not None:
            random.seed(seed)
        random.shuffle(self.questions)

    def shuffle_question(self, start_index: int, end_index: int, seed: int = None):
        """
        shuffle question from start_index to end_index
        """
        if seed is not None:
            random.seed(seed)

        copy = self.questions[start_index:end_index]
        random.shuffle(copy)
        self.questions[start_index:end_index] = copy

    def shuffle_answer(self, not_shuffle=None, seed: int = None):
        """
        Shuffle question answer

        :param not_shuffle: Index list of questions that do not shuffle answers
        :param seed: random seed
        """
        if not_shuffle is None:
            not_shuffle = []

        if seed is not None:
            random.seed(seed)

        for index, question in enumerate(self.questions):
            if index not in not_shuffle:
                question.shuffle_answer()


class ExamExportType(Enum):
    """Class present all export exam options: PDF and TEX"""
    PDF = 0
    TEX = 1



import os


class LatexPaper:
    """This base class is used to export exam in tex and pdf file"""
    def get_latex_string(self) -> str:
        pass

    def export_tex_file(self, file_dir: str):
        """Export to .tex file"""
        file = open(f"{file_dir}.tex", "w")
        file.write(self.get_latex_string())
        file.close()

    def export_pdf_file(self, file_dir: str):
        """Export to pdf file"""
        self.export_tex_file(file_dir)
        os.system(f"pdflatex {file_dir}.tex")



from typing import List

from jinja2env import jinja_env
from pytexexam.latexpaper import LatexPaper
from pytexexam.question import Question


class LatexExamPaper(LatexPaper):
    """This class is used to create an exam paper"""
    def __init__(self):
        self.preamble = ""
        self.header = ""
        self.questions: List[Question] = list()
        self.footer = ""
        self.question_translation = "Question"

    def get_latex_string(self) -> str:
        """Generate latex code"""
        question_str = ""
        for question in self.questions:
            question_str += (question.print_question_latex() + "\n\n")

        return jinja_env.get_template("exam.tex").render(
            question_theorem=self.question_translation,
            user_preamble=self.preamble,
            exam_header=self.header,
            question_str=question_str,
            exam_footer=self.footer
        )


class LatexExamAnswer(LatexPaper):
    """This class is used to create an answer keys"""
    def __init__(self):
        self.preamble = ""
        self.header = ""
        self.questions: List[Question] = list()
        self.footer = ""

    def get_latex_string(self) -> str:
        """Generate latex code"""
        return jinja_env.get_template("answer.tex").render(
            user_preamble=self.preamble,
            exam_header=self.header,
            questions=self.questions,
            exam_footer=self.footer
        )


class LatexExamSolution(LatexPaper):
    """Generate solution paper for this exam"""
    def __init__(self):
        self.preamble = ""
        self.header = ""
        self.questions = list()
        self.footer = ""
        self.question_translation = "Question"

    def get_latex_string(self) -> str:
        """Generate latex code"""
        solution_str = ""
        for question in self.questions:
            solution_str += (question.print_solution_latex() + "\n\n")
        return jinja_env.get_template("exam.tex").render(
            question_theorem=self.question_translation,
            user_preamble=self.preamble,
            exam_header=self.header,
            question_str=solution_str,
            exam_footer=self.footer
        )



import string
from random import SystemRandom

from pytexexam.answer import Answer
from jinja2env import jinja_env
from typing import List


class Question:
    """
    This class represents one question on the test.
    """
    def __init__(self, question: str, answers: List[str], true_answer: str, solution: str, answer_column: int):
        self.question: str = question
        """Content of the question."""
        self.answers: List[Answer] = self.__get_answer_list(answers, true_answer)
        """Question answers"""
        self.answer_column = answer_column
        """Number of columns for which the answer will be presented."""
        self.solution = solution
        """Solution of the question"""

    @staticmethod
    def __get_answer_list(answers: List[str], true_answer: str) -> List[Answer]:
        """
        Generate a list of answer object from answer and true answer key
        :param answers: question answer list.
        :param true_answer: answer key of true answer.
        :return: a list of answer object.
        """
        answer_key = Question.__get_answer_key()
        answer_list_size = min(len(answer_key), len(answers))
        answer_list: List[Answer] = []
        for i in range(0, answer_list_size):
            answer_list.append(Answer(answer_key[i], answers[i], answer_key[i] == true_answer))
        return answer_list

    @staticmethod
    def __get_answer_key() -> List[str]:
        """
        get list of alphabet character. (to use it as answer key)
        :return: Alphabet character list.
        """
        return list(string.ascii_uppercase)

    def shuffle_answer(self):
        """Shuffle answer list"""
        SystemRandom().shuffle(self.answers)

    def get_true_answer_key(self) -> str:
        """Get answer key of true answer"""
        true_answer = ""
        for answer in self.answers:
            if answer.is_true_answer:
                true_answer += answer.answer_key
        return true_answer

    def print_question_latex(self) -> str:
        """generate latex code for this question"""
        column_size = 1 / self.answer_column
        table_column = rf"*{{{self.answer_column}}}{{S{{m{{\dimexpr{column_size}\linewidth-2\tabcolsep\relax}}}}}}"

        answer_string = ""
        for i, answer in enumerate(self.answers):
            seperator = "\\\\\n" if((i+1) % self.answer_column == 0) else "&"
            answer_string += f"\\textbf{{{answer.answer_key}}}. {answer.answer}. {seperator} "

        return jinja_env.get_template("mcq.tex").render(
            question=self.question,
            table_column=table_column,
            answer_string=answer_string
        )

    def print_solution_latex(self) -> str:
        """Generate latex code to print question and solution"""
        return jinja_env.get_template("mcqsolution.tex").render(
            question=self.print_question_latex(),
            solution=self.solution
        )



from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='pytexexam',
    version='2.1.3',
    packages=['pytexexam'],
    include_package_data=True,
    url='https://github.com/vungocbinh2009/pytexexam',
    license='Apache License, Version 2.0',
    author='binh',
    author_email='vungocbinh@protonmail.com',
    description='A simple library to create latex exam in python ',
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=['jinja2'],
)
